<?php
function node_recommendation_views_tables() {
  $tables['node_similarity'] = array(
    'name' => 'cre_similarity_matrix',
    'provider' => 'internal',
    'join' => array(
      'left' => array(
        'table' => 'node',
        'field' => 'nid',
      ),
      'right' => array(
        'field' => 'content_id1',
      ),
      'extra' => array(
        /* we only care about things where both are nodes... not totally true...
         * but because Views 1.x only is for nodes nothing else will 
         * matter with the votingapi table */
        'content_type1' => 'node',
        'content_type2' => 'node',
      ),
    ),
  );

  $tables['node_recommendation_user'] = array(
    'name' => 'node_recommendations',
    'provider' => 'internal',
    'join' => array(
      'left' => array(
        'table' => 'node',
        'field' => 'nid',
      ),
      'right' => array(
        'field' => 'nid',
      ),
      'extra' => array(
        'uid' => '***CURRENT_USER***'
      ),
    ),
    'fields' => array(
      'recommenadation_score' => array(
        'name' => t('Content Recommendation: Predicated Value'),
        'field' => 'score',
        'help' => t('Predicted score for the logged in user on this node'),
      ),
    ),
    'sorts' => array(
      'recommendation_score' => array(
        'name' => t('Content Recommendation: Recommendation Score'),
        'handler' => 'node_recommendation_score_handler',
        'help' => t('Sort by the recommendation score for the current user'),
      ),
    ),
  );
  
  return $tables;
}

function node_recommendation_score_handler($op, &$query, $sortinfo, $sort) {
  $query->add_orderby('node_recommendation_user', 'score', $sort['sortorder']);
  // perhaps a better join prevents this
  $query->add_where("node_recommendation_user.score <> ''");
}

function node_recommendation_views_arguments() {
  $arguments = array(
    'similarity' => array(
      'name' => t('Content Recommendation: Similar node'),
      'handler' => 'node_recommendation_similarity',
      'help' => t('Calculates the similarity scores between this nid and every other id and sorts by that similarity'),
    ),
  );
  return $arguments;
}

function node_recommendation_similarity($op, &$query, $argtype, $arg = '') {
  switch($op) {
    case 'filter':
      $base_nid = intval($arg);
      $query->ensure_table('node_similarity');
      $field = 'ABS(node_similarity.sum / node_similarity.count)';
      $query->add_orderby('', $field, 'ASC', 'avg_difference');
      //WHERE d.content_id2 = 61 AND d.content_id1 <> 61
      $query->add_where('node_similarity.content_id2 = %d', $base_nid);
      $query->add_where('node_similarity.content_id1 <> %d', $base_nid);
      
  }
}

function node_recommendation_views_default_views() {
  $view = new stdClass();
  $view->name = 'node_recommendations';
  $view->description = '';
  $view->access = array (
);
  $view->view_args_php = '';
  $view->page = TRUE;
  $view->page_title = '';
  $view->page_header = '';
  $view->page_header_format = '1';
  $view->page_footer = '';
  $view->page_footer_format = '1';
  $view->page_empty = '';
  $view->page_empty_format = '1';
  $view->page_type = 'teaser';
  $view->url = 'recommendations';
  $view->use_pager = TRUE;
  $view->nodes_per_page = '10';
  $view->sort = array(
    array(
      'tablename' => 'node_recommendation_user',
      'field' => 'recommendation_score',
      'sortorder' => 'DESC',
      'options' => '',
    ),
  );
  $view->argument = array(
  );
  $view->field = array(
    array(
      'tablename' => 'node',
      'field' => 'title',
      'label' => 'Title',
      'handler' => 'views_handler_field_nodelink',
      'options' => 'link',
    ),
  );
   $view->filter = array(
    array(
      'tablename' => 'node',
      'field' => 'status',
      'operator' => '=',
      'options' => '',
      'value' => '1',
    ),
  );
  $view->exposed_filter = array(
  );
  $view->requires = array(node_recommendation_user, node);
  $views[$view->name] = $view;
  
     $view = new stdClass();
  $view->name = 'similar_nodes';
  $view->description = '';
  $view->access = array (
);
  $view->view_args_php = '';
  $view->page = TRUE;
  $view->page_title = 'Similar nodes';
  $view->page_header = '';
  $view->page_header_format = '1';
  $view->page_footer = '';
  $view->page_footer_format = '1';
  $view->page_empty = '';
  $view->page_empty_format = '1';
  $view->page_type = 'teaser';
  $view->url = 'node/$arg/similar';
  $view->use_pager = TRUE;
  $view->nodes_per_page = '10';
  $view->menu = TRUE;
  $view->menu_title = 'Similar Nodes';
  $view->menu_tab = TRUE;
  $view->menu_tab_weight = '0';
  $view->menu_tab_default = FALSE;
  $view->menu_tab_default_parent = NULL;
  $view->menu_tab_default_parent_type = 'tab';
  $view->menu_parent_tab_weight = '0';
  $view->menu_parent_title = '';
  $view->sort = array(
  );
  $view->argument = array(
    array(
      'type' => 'similarity',
      'argdefault' => '1',
      'title' => '',
      'options' => '',
      'wildcard' => '',
      'wildcard_substitution' => '',
    ),
  );
  $view->field = array(
  );
  $view->filter = array(
    array(
      'tablename' => 'node',
      'field' => 'status',
      'operator' => '=',
      'options' => '',
      'value' => '1',
    ),
  );
  $view->exposed_filter = array(
  );
  $view->requires = array(node);
  $views[$view->name] = $view;



  return $views;
}
